﻿Option Explicit On


Module Module1

Public Const WHDR_DONE = &H1         '  done bit

Structure WAVEHDR
   Dim lpData As Integer
   Dim dwBufferLength As Integer
   Dim dwBytesRecorded As Integer
   Dim dwUser As Integer
   Dim dwFlags As Integer
   Dim dwLoops As Integer
   Dim lpNext As Integer
   Dim Reserved As Integer
End Structure

Structure WAVEFORMAT
   Dim wFormatTag As Short
   Dim nChannels As Short
   Dim nSamplesPerSec As Integer
   Dim nAvgBytesPerSec As Integer
   Dim nBlockAlign As Short
   Dim wBitsPerSample As Short
   Dim cbSize As Short
End Structure

Declare Function GlobalAlloc Lib "kernel32" (ByVal wFlags As Integer, ByVal dwBytes As Integer) As Integer
Declare Function GlobalLock Lib "kernel32" (ByVal hmem As Integer) As Integer
Declare Function GlobalFree Lib "kernel32" (ByVal hmem As Integer) As Integer
'Declare Sub CopyStructFromPtr Lib "kernel32" Alias "RtlMoveMemory" (ByRef struct As Any, ByVal ptr As Integer, ByVal cb As Integer)
Declare Sub CopyStructFromPtr Lib "kernel32" Alias "RtlMoveMemory" (ByRef struct As Short, ByVal ptr As Integer, ByVal cb As Integer)
Declare Function waveInOpen Lib "winmm.dll" (ByRef lphWaveIn As Integer, ByVal uDeviceID As Integer, ByRef lpFormat As WAVEFORMAT, ByVal dwCallback As Integer, ByVal dwInstance As Integer, ByVal dwFlags As Integer) As Integer
Declare Function waveInStart Lib "winmm.dll" (ByVal hWaveIn As Integer) As Integer
Declare Function waveInStop Lib "winmm.dll" (ByVal hWaveIn As Integer) As Integer
Declare Function waveInAddBuffer Lib "winmm.dll" (ByVal hWaveIn As Integer, ByRef lpWaveInHdr As WAVEHDR, ByVal uSize As Integer) As Integer
Declare Function waveInReset Lib "winmm.dll" (ByVal hWaveIn As Integer) As Integer
Declare Function waveInPrepareHeader Lib "winmm.dll" (ByVal hWaveIn As Integer, ByRef lpWaveInHdr As WAVEHDR, ByVal uSize As Integer) As Integer
Declare Function waveInGetErrorText Lib "winmm.dll" Alias "waveInGetErrorTextA" (ByVal err As Integer, ByVal lpText As String, ByVal uSize As Integer) As Integer
Declare Function waveInClose Lib "winmm.dll" (ByVal hWaveIn As Integer) As Integer
Declare Function waveInUnprepareHeader Lib "winmm.dll" (ByVal hWaveIn As Integer, ByRef lpWaveInHdr As WAVEHDR, ByVal uSize As Integer) As Integer

Public i As Short, j As Short, rc As Integer, msg As String = Space$(200), hWaveIn As Integer
Public format As WAVEFORMAT
Public hmem As Integer
Public inHdr As WAVEHDR
Public Const DEVICEID = 0
Public fRecording As Boolean
Public BUFFER_SIZE As Short

'When the number of FFT points is < 512 and samplerates
'are < 22050 are used the program has to wait for input.
'A circular buffer of sufficient size may be of value
'but I don't know how to implement it.

Function StartInput() As Boolean

    If fRecording Then
        StartInput = True
        Exit Function
    End If

    format.wFormatTag = 1
    format.nChannels = 1
    format.wBitsPerSample = 16
    format.nSamplesPerSec = 48000    'Gets the sample rate from the main form
    format.nBlockAlign = format.nChannels * format.wBitsPerSample / 8
    format.nAvgBytesPerSec = format.nSamplesPerSec * format.nBlockAlign
    format.cbSize = 0

    hmem = GlobalAlloc(&H40, BUFFER_SIZE)
    inHdr.lpData = GlobalLock(hmem)
    inHdr.dwBufferLength = BUFFER_SIZE
    inHdr.dwFlags = 0
    inHdr.dwLoops = 0

    rc = waveInOpen(hWaveIn, DEVICEID, format, 0, 0, 0)
    If rc <> 0 Then
      waveInGetErrorText(rc, msg, Len(msg))
      MsgBox(msg)
      StartInput = False
      Exit Function
    End If

    rc = waveInPrepareHeader(hWaveIn, inHdr, Len(inHdr))
    If (rc <> 0) Then
       waveInGetErrorText(rc, msg, Len(msg))
       MsgBox(msg)
    End If

    rc = waveInAddBuffer(hWaveIn, inHdr, Len(inHdr))
    If (rc <> 0) Then
       waveInGetErrorText(rc, msg, Len(msg))
       MsgBox(msg)
    End If

    fRecording = True
    rc = waveInStart(hWaveIn)
    StartInput = True

End Function

' Stop receiving audio input on the soundcard
Sub StopInput()
    fRecording = False
    waveInReset(hWaveIn)
    waveInStop(hWaveIn)
    waveInUnprepareHeader(hWaveIn, inHdr, Len(inHdr))
    GlobalFree(hmem)
    waveInClose(hWaveIn)
End Sub

Sub GetMono16Sample(ByVal sample As Integer, ByRef leftVol As Double)
   Dim sample16 As Short
   Dim ptr As Integer

   ptr = sample * format.nBlockAlign + inHdr.lpData
   CopyStructFromPtr(sample16, ptr, 2)
   leftVol = sample16 / 32768

End Sub

'The following code may be from the original program author
'but it appears to be a relatively generic adaptation.
'This routine imposes a speed bottleneck when the number
'of FFT points is > 512.

'THE FAST FOURIER TRANSFORM
'Upon entry, N% contains the number of points in the DFT,
'REX[ ] and IMX[ ] contain the real and imaginary parts of the input.
'Upon return, REX[ ] and IMX[ ] contain the DFT output.
'All signals run from 0 to N-1.

Sub FFT(ByVal N As Short, ByVal REX() As Double, ByVal IMX() As Double)

    Const Pi = 3.14159
    Dim NM1 As Short
    Dim ND2 As Short
    Dim M As Short
    Dim j As Short
    Dim K As Short
    Dim L As Short
    Dim LE As Short
    Dim LE2 As Short
    Dim JM1 As Short
    Dim i As Short
    Dim IP As Short
    Dim TR As Double
    Dim TI As Double
    Dim UR As Double
    Dim UI As Double
    Dim SR As Double
    Dim SI As Double

    NM1 = N - 1
    ND2 = N / 2
    M = CInt(Math.Log(N) / Math.Log(2))
    j = ND2

    For i = 1 To N - 2                    'Bit reversal sorting
      If i >= j Then GoTo S1
      TR = REX(j)
      TI = IMX(j)
      REX(j) = REX(i)
      IMX(j) = IMX(i)
      REX(i) = TR
      IMX(i) = TI
S1:   K = ND2
S2:   If K > j Then GoTo s3
      j = j - K
      K = K / 2
      GoTo S2
s3:   j = j + K
    Next

    For L = 1 To M                          'Loop for each stage
        LE = CInt(2 ^ L)
        LE2 = LE / 2
        UR = 1
        UI = 0
        SR = Math.Cos(Pi / LE2)                 'Calculate sine & cosine values
        SI = -Math.Sin(Pi / LE2)

        For j = 1 To LE2                    'Loop for each sub DFT
             JM1 = j - 1
             For i = JM1 To NM1 Step LE     'Loop for each butterfly
                   IP = i + LE2
                   TR = REX(IP) * UR - IMX(IP) * UI   'Butterfly calculation
                   TI = REX(IP) * UI + IMX(IP) * UR
                   REX(IP) = REX(i) - TR
                   IMX(IP) = IMX(i) - TI
                   REX(i) = REX(i) + TR
                   IMX(i) = IMX(i) + TI
             Next
             TR = UR
             UR = TR * SR - UI * SI
             UI = TR * SI + UI * SR
        Next
    Next

End Sub





End Module
